﻿
Public Class ListViewFlickerFree
    Inherits System.Windows.Forms.ListView

    Public Sub New()
        MyBase.New()
    End Sub

    Public Sub RefreshBackground()
        EraseBkgEnabled = True
        Refresh()
    End Sub

    ' =====================================================================================
    '  Owner Drawn - ColumnHeaders
    ' =====================================================================================
    Public Header_BackColor1 As Color = Color.FromArgb(255, 255, 200)
    Public Header_BackColor2 As Color = Color.FromArgb(200, 200, 200)
    Public Header_ForeColor As Color = Color.FromArgb(50, 40, 0)
    Public Header_ShadowColor As Color = Color.FromArgb(0, 120, 120, 120)
    Public Header_Font As Font = New Font("Tahoma", 11, FontStyle.Regular)
    Public Header_TextVerticalPosition As Int32 = 0
    Public Header_TextAlignments(-1) As StringAlignment


    ' ===============================================================================================
    '  DISABLE ERASE BACKGROUND
    ' ===============================================================================================
    Const WM_ERASEBKGND As Int32 = &H14

    'Const WM_HSCROLL As Int32 = &H114
    'Const WM_VSCROLL As Int32 = &H115
    'Const WM_NCLBUTTONUP As Int32 = &HA2
    'Const WM_NCHITTEST As Int32 = &H84
    'Const WM_NCMOUSELEAVE As Int32 = &H2A2

    Const WM_KEYUP As Int32 = &H101
    Const WM_CAPTURECHANGED As Int32 = &H215

    Const LVM_INSERTITEMA As Int32 = &H1007
    Const LVM_INSERTITEMW As Int32 = &H104D
    Const LVM_INSERTGROUP As Int32 = &H1091
    Const LVM_DELETEITEM As Int32 = &H1008
    Const LVM_DELETEALLITEMS As Int32 = &H1009

    Private EraseBkgEnabled As Boolean = True

    <DebuggerStepThrough()> _
    Protected Overrides Sub WndProc(ByRef m As Message)
        If Me.Items.Count = 0 Then EraseBkgEnabled = True
        'Debug.Print(m.Msg.ToString)
        Select Case m.Msg
            Case WM_ERASEBKGND
                If EraseBkgEnabled Then
                    EraseBkgEnabled = False
                    MyBase.WndProc(m)
                Else
                    m.Result = IntPtr.Zero
                End If
            Case LVM_INSERTGROUP, LVM_INSERTITEMA, LVM_INSERTITEMW, LVM_DELETEITEM, LVM_DELETEALLITEMS
                MyBase.WndProc(m)
                EraseBkgEnabled = True
                ' ------------------------- refresh background when KeyUp the mouse releases the scroll bar
            Case WM_KEYUP, WM_CAPTURECHANGED
                MyBase.WndProc(m)
                EraseBkgEnabled = True
                Invalidate()
            Case Else
                MyBase.WndProc(m)
        End Select
    End Sub

    ' =====================================================================================
    '  EVENTS
    ' =====================================================================================
    Private Sub ListViewFlickerFree_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.SelectedIndexChanged
        EraseBkgEnabled = True
    End Sub
    Private Sub ListViewFlickerFree_Layout(ByVal sender As Object, ByVal e As System.Windows.Forms.LayoutEventArgs) Handles Me.Layout
        EraseBkgEnabled = True
    End Sub
    Private Sub ListViewFlickerFree_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        EraseBkgEnabled = True
        Invalidate()
    End Sub


    ' ==============================================================================================
    '  THE FOLLOWING PARTS ARE USED ONLY FOR A LIST-VIEW WITH LABEL-EDIT ENABLED
    ' ==============================================================================================
    Private Sub ListViewFlickerFree_BeforeLabelEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.LabelEditEventArgs) Handles Me.BeforeLabelEdit
        EraseBkgEnabled = True
    End Sub
    Private Sub ListViewFlickerFree_AfterLabelEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.LabelEditEventArgs) Handles Me.AfterLabelEdit
        EraseBkgEnabled = True
    End Sub

    ' ==============================================================================================
    '  The following part is used to detect KeyDown, KeyUp and KeyPress while editing LABEL
    ' ==============================================================================================
    Private Declare Function SendMessage Lib "user32.dll" Alias "SendMessageW" _
                                                            (ByVal hWnd As IntPtr, _
                                                             ByVal msg As Integer, _
                                                             ByVal wp As IntPtr, _
                                                             ByVal lp As IntPtr) As IntPtr

    Private Label As LabelTextBox

    Protected Sub OnLabelKeyDown(ByVal e As KeyEventArgs)
        EraseBkgEnabled = True
    End Sub
    Protected Sub OnLabelKeyUp(ByVal e As KeyEventArgs)
        EraseBkgEnabled = True
    End Sub
    Protected Sub OnLabelKeyPress(ByVal e As KeyPressEventArgs)
        EraseBkgEnabled = True
    End Sub

    Private Sub BeforeLabelEdit_ForKeyEvents(ByVal sender As Object, ByVal e As System.Windows.Forms.LabelEditEventArgs) Handles Me.BeforeLabelEdit
        Label = New LabelTextBox(Me)
        Const LVM_GETEDITCONTROL As Integer = 4096 + 24
        Dim hdl As IntPtr = SendMessage(Me.Handle, LVM_GETEDITCONTROL, IntPtr.Zero, IntPtr.Zero)
        Me.Label.AssignHandle(hdl)
    End Sub
    Private Sub AfterLabelEdit_ForKeyEvents(ByVal sender As Object, ByVal e As System.Windows.Forms.LabelEditEventArgs) Handles Me.AfterLabelEdit
        Me.Label.ReleaseHandle()
    End Sub

    Private Class LabelTextBox
        Inherits NativeWindow

        Private Parent As ListViewFlickerFree

        Public Sub New(ByVal parent As ListViewFlickerFree)
            MyBase.New()
            Me.Parent = parent
        End Sub

        Protected Overrides Sub WndProc(ByRef m As Message)
            Select Case m.Msg
                Case 256
                    Dim args As KeyEventArgs = New KeyEventArgs(CType(m.WParam, Keys))
                    Me.Parent.OnLabelKeyDown(args)
                    If args.Handled Then Return
                Case 257
                    Dim args As KeyEventArgs = New KeyEventArgs(CType(m.WParam, Keys))
                    Me.Parent.OnLabelKeyUp(args)
                Case 258
                    Dim args As KeyPressEventArgs = New KeyPressEventArgs(ChrW(m.WParam.ToInt32))
                    Me.Parent.OnLabelKeyPress(args)
                    If args.Handled Then Return
            End Select
            MyBase.WndProc(m)
        End Sub
    End Class

End Class
